//
// USB-Linker for GBx
//
// (C) 2003 Mootan

// 2003.02.10 Mootan
//			First coding.
//
//
// TODO:
// dubble buffering Endpoint
// optimize SUB ISR overhead
//
//

#include "..\_common\_startup.c"
#include "..\_common\ezusb.h"
#include "..\_common\frame.h"
#include "..\_common\bit.h"

/////////////////////////////// User Oriented ////////////////////////////////////////
//
#include "gbx_boot.h"
#include "target_cmd.h"

// Assember function
//
// Hint:
// Assembler function can be used.
//
extern void reset_port();
extern void set_ai(unsigned char);
extern void write_data(unsigned char);
extern unsigned char read_data();
//
//
// user definition
//
#define USING_ENDPOINT_TOTAL	0x04
#define TM0OVER_FLAG			bmBIT5


//
// EndPoint Counter initial value
// Maximum 64 as Buffer size of each EndPoint
//
#define INIT_EP01_COUNT	1
#define INIT_EP05_COUNT	1
#define INIT_EP03_COUNT	1

// Buffer handlng unit is step value for each EndPoint handling
#define BUFFER_HANDLING_UNIT_EP01	1
#define BUFFER_HANDLING_UNIT_EP05	1

#define BUFFER_HANDLING_UNIT_EP04	1

// 
// Hint:
// Flags for each EndPoint
// OUTBUF7 is used .
// Boundary is 8byte for each EndPoint.
//
struct EPCTRL_STRUC {

// General
	unsigned int	wValue;
	unsigned int	wIndex;
	unsigned char	w_dummy[4];

// Control EndPoint
	unsigned char	cep_count;
	unsigned char	cep_dat;
	unsigned char	cep_stat;
	unsigned char	cep_dummy[5];

// Interrupt EndPoint
	unsigned char	iep_count;
	unsigned char	iep_dummy[7];

// BulkOut EndPoint
	unsigned char	bout_countEP2;
	unsigned char	bout_countEP4;
	unsigned char	bout_dummy[6];

// BulkIn EndPoint
	unsigned char	bin_countEP3;
	unsigned char	bin_countEP5;
	unsigned char	bin_dummy[6];

};

xdata at 0x7b40 volatile struct EPCTRL_STRUC  EPCTRL;	// OUT7BUF is used.


//
// EndPoint assignment
//
// Endpoint #0 : EP0 : Control Endpoint
// Endpoint #1 : IEP : Interrupt IN (FIFO:16bytes)
// Endpoint #2 : BIN : Bulk IN
// Endpoint #3 : BOUT: Bulk OUT
//

//
// Device desciptor
//
const unsigned char dev_desc[DEVICE_DISC_LEN]  = {
	0x12,			//  bLength				: 0x12
	0x01, 			//  bDescriptorType		: 0x01		Device desciptor
	0x00, 0x01,		//  bcdUSB				: 0x0100	USB Ver.1.0
	0x00,			//  bDeviceClass		: 0x00		Set for each Interface
	0x00,			//  bDeviceSubClass		: 0x00		Not used
	0x00, 			//  bDeviceProtocol		: 0x00		Not used
	0x40,			//  bMaxPacketSize0		: 0x08		Maximum Packet size of EP0
	VENDER_ID_LOW,	//  idVender
	VENDER_ID_HIGH,	//  idVendor
	PRODUCT_ID_LOW, //  idProduct
	PRODUCT_ID_HIGH,//  idProduct
	0x01, 0x00,		//  bcdDevice			: 0x0001	Device Release Number
	0x00, 			//  iManufacture		: 0x00		Not used
	0x00, 			//  iProduct			: 0x00		Not used
	0x00, 			//  iSerialNumber		: 0x00		Not used
	0x01			//  bNumConfiguraions	: 0x01		Configuration is single
};

//
// Configuration Desciptor
//
const unsigned char config_desc[CONFIG_DISC_LEN] = {
	0x09,			//  bLength				: 0x09
	0x02,			//  bDescriptorType		: 0x02	Configuration desciptor
//	0x35, 0x00,		//  wTotalLength		: 53	Total lenght of Interface number and EndPoints
					//								CFG(9) + IF(9) x 1 + EP(7) x 5 = 53
	0x2E, 0x00,		//  wTotalLength		: 46	Total lenght of Interface number and EndPoints
	0x01,			//  bNumberIntefaces	: 0x01	Inerface is single
	0x01,			//  bConfigurationValue	: 0x01	Therefore 0x01
	0x00,			//  iConfiguration		: 0x00	Not used
	0x80,			//  bmAttributes		: 0x80	Bus Power device and no remote wakeup
	250				//  MaxPower			: 250	Consumption current 500mA is maximun for temporary setting.
};

//
// Interface Descriptor 1
//
// Interface #1
//

const unsigned char interface_desc[INTERFACE_DISC_LEN] = {
	0x09,		//  bLength					: 0x09
	0x04,		//  bDescriptorType			: 0x04	Interfacd descriptor
	0x00,		//  bInterfaceNumber		: 0x00	Interface #1
	0x00,		//  bAlternateSetting		: 0x00	Alternate setting is not used
//	0x05,		//  bNumEndpointes			: 0x05	Number of used EndPoint as 5
	0x04,		//  bNumEndpointes			: 0x02	Number of used EndPoint as 4
	0xff,		//  bInterfaceClass			: 0xff	Vendor definition class
	0x00,		//  bInterfaceSubClass		: 0x00	Sub-class is not used
	0xff,		//  bInterfaceProtocol		: 0xff	Protocol is defined by vendor
	0x00 		//  iInterface				: 0x00	Not used
};

//
// EndPoint descriptor
//
// EndPoint #1 (Interrupt in)
//  bLength					: 0x07
//  bDescriptorType			: 0x05		EndPoint descriptor
//  bEndpointAddress		: 0x81		Direction is In. Address #1
//  bmAttributes			: 0x03		Interrupt mode
//  wMaxPacketSize			: 0x0040	Maximum packet size is 64byte
//  bInterval				: 10		Polling time is every 10ms
//
//
// EndPoint #2 and #3 are not used this program.
//
//
// EndPoint #2 (BulkOut)
//  bLength					: 0x07
//  bDescriptorType			: 0x05		EndPoint descriptor
//  bEndpointAddress		: 0x02		Direction is out. Address #2
//  bmAttributes			: 0x02		Bulk mode
//  wMaxPacketSize			: 0x0040	Maximum packet size is 64byte.
//  bInterval				: 0x00		No meaning in Bulk mode.
//
// EndPoint #3 (BulkIn)
//  bLength					: 0x07
//  bDescriptorType			: 0x05		EndPoint descrptor
//  bEndpointAddress		: 0x83		Direction is in. Addres #3
//  bmAttributes			: 0x02		Bulk mode
//  wMaxPacketSize			: 0x0040	Maximum packet size is 64byte.
//  bInterval				: 0x00		No meaning in Bulk mode.
//
//
// EndPoint #4 (BulkOut)
//  bLength					: 0x07
//  bDescriptorType			: 0x05		EndPoint descriptor
//  bEndpointAddress		: 0x04		Direction is out. Address #4
//  bmAttributes			: 0x02		Bulk mode
//  wMaxPacketSize			: 0x0040	Maximum packet size is 64byte.
//  bInterval				: 0x00		No meaning in Bulk mode.
//
// EndPoint #5 (BulkIn)
//  bLength					: 0x07
//  bDescriptorType			: 0x05		EndPoint descriptor
//  bEndpointAddress		: 0x85		Direction is in Address #5
//  bmAttributes			: 0x02		Bulk mode
//  wMaxPacketSize			: 0x0040	Maximum packet size is 64byte
//  bInterval				: 0x00		No meaning in Bulk mode.

unsigned char endpoint_desc[USING_ENDPOINT_TOTAL][ENDPOINT_DISC_LEN] = {
	{0x07, 0x05, 0x81, 0x03, 0x40, 0x00, 10},	// Interrupt-IN Endpoint
	{0x07, 0x05, 0x02, 0x02, 0x40, 0x00, 0x00},	// Bulk-OUT Endpoint
	{0x07, 0x05, 0x83, 0x02, 0x40, 0x00, 0x00},	// Bulk-IN Endpoint
	{0x07, 0x05, 0x04, 0x02, 0x40, 0x00, 0x00},	// Bulk-OUT Endpoint
//	{0x07, 0x05, 0x85, 0x02, 0x40, 0x00, 0x00}	// Bulk-IN Endpoint
};

// Hint:
// Register init at startup
// This routine will be modified for your purpose.
// EndPoint handling will be changed if you use other EndPoint.
//
void register_init(void)
{

	CKCON =0x00;		// external bus 2cycle

 	PORTBCFG	= 0;	//
	PORTCCFG	= 0;	//
	reset_port();		// reset GBx probe port
	OUTA		= 0;
	OEA			= 0;

	USBBAV		= 0x00;	// Not Break & Non-AutoVector Mode

//////////////////////////////
//	USBPAIR  = 0x10;	// EP4OUT pair
//////////////////////

	USBIEN		= 0x01;
//	IN07VAL		= 0x2b;	// EP0/EP1/EP3/EP5 Enable
	IN07VAL		= 0x09;	// EP0/EP3 Enable
//	OUT07VAL	= 0x15;	// EP0/EP2/EP4 Enable
	OUT07VAL	= 0x11;	// EP0/EP4 Enable
	IN07IEN		= 0xff;
//	OUT07IEN	= 0xff;
	OUT07IEN	= 0x11;

	IN1CS		|= INnCS_BSY;
	IN5CS		|= INnCS_BSY;

	EIE			= 0xe1;	// USB Interrupt Enable
	EIP			= 0;	// Normal Priority

	IE			= 0x80;	// Interrupt Enable 
}


//
// Response for GET_DESCRIPTOR(CONFIGURATION)
// 
void setup_usb_inf(void) 
{ 
	unsigned int p, i, j; 
	p=0;

	// 
	for (i = 0; i < DEVICE_DISC_LEN; i++){ 
 		IN7BUF[p] = dev_desc[i]; 
  		p++; 
	}

	//
	for (i = 0; i < CONFIG_DISC_LEN; i++){ 
		IN7BUF[p] = config_desc[i]; 
		p++; 
	} 

	//
	for (i = 0; i < INTERFACE_DISC_LEN; i++){ 
		IN7BUF[p] = interface_desc[i]; 
		p++; 
	}

	for (j = 0; j < USING_ENDPOINT_TOTAL; j++){ 
		for (i = 0; i < ENDPOINT_DISC_LEN; i++){ 
   			IN7BUF[p] = endpoint_desc[j][i]; 
   			p++;
		}
	}
}
//
//
void wait(unsigned long ticks)
{
	while(ticks--);
}
//

///////////////////////////////////////////////////////////
// GB-Xchanger basic functions

void set_ai_data(unsigned char _ai, unsigned char _data)
{
	set_ai(_ai);
	write_data(_data);
}

void set_adr(unsigned int adr)
{
	set_ai_data(0, (adr & 0xFF));	// a[7..0]
	set_ai_data(1, (adr >> 8));		// a[15..8]
	set_ai(3);					// default write mode
}

void out_byte(unsigned char d)
{
	set_ai_data(3, d);
}

void out_byte_eeprom(unsigned char d)
{
	set_ai_data(2, 0x82);		// wei enable
	out_byte(d);
	set_ai_data(2, 0x80);		// wei disable
	set_ai(3);					// default write mode
}

void init_port()
{
	set_ai_data(2, 0);			// rst=0, wei=0(dis.), rdi=0(dis.)
	set_ai_data(2, 0x80);		// rst=1, wei=0(dis.), rdi=0(dis.)
}

void out_adr_data_32k(unsigned int adr, unsigned char d)
{
	set_adr(adr);				// write adr:data
	out_byte_eeprom(d);			// write data to eeprom
}

unsigned char xh, h, m, l;
void split_adr(unsigned long adr)
{
	l  =  adr		 & 0xFF;	// a7~a0
	m  = (adr >>  8) & 0x3F;	// a13~a8
	h  = (adr >> 14) & 0xFF;	// a21~a13
	xh = (adr >> 22) & 0x07;	// max. 256Mbit
	if ( h | xh ) m |= 0x40;	// >bank 0
}

void set_adr_to_bank()
{
	set_ai_data(2, 0x80);		// disable wr/rd inc.
	set_adr(0x3000);			// write 3000:sh
	write_data(xh);				// set rom bank extend value
	set_adr(0x2000);			// write 2000:h
	write_data(h);				// set rom bank value
	set_ai_data(1, m);			// a[15..8]
	set_ai_data(0, l);			// a[7..0]
}

void out_adr_data(unsigned long adr, unsigned char d)	// real address
{
	split_adr(adr);
	set_adr_to_bank();
	out_byte_eeprom(d);			// write data to eeprom
}

const unsigned char check_header[48] = {
	0xce,0xed,0x66,0x66,0xcc,0x0d,0x00,0x0b,
	0x03,0x73,0x00,0x83,0x00,0x0c,0x00,0x0d,
	0x00,0x08,0x11,0x1f,0x88,0x89,0x00,0x0e,
	0xdc,0xcc,0x6e,0xe6,0xdd,0xdd,0xd9,0x99,
	0xbb,0xbb,0x67,0x63,0x6e,0x0e,0xec,0xcc,
	0xdd,0xdc,0x99,0x9f,0xbb,0xb9,0x33,0x3e
};

void check_eeprom(void)
{
	unsigned char i;
	for ( i = 0; i < 0x50; i++ ) {
		set_adr((unsigned int)i + 0x100);
		IN5BUF[i] = read_data();
	}
	for ( i = 0; i < 48; i++ )
		if ( IN5BUF[i+4] != check_header[i] ) break;
	if ( i == 48 ) {
		gCartType = IN5BUF[0x47];
		i		  = IN5BUF[0x48];
		switch ( i ) {
		case 0x52:
			gRomSize = 16*1024 * 72;
			break;
		case 0x53:
			gRomSize = 16*1024 * 80;
			break;
		case 0x54:
			gRomSize = 16*1024 * 96;
			break;
		default:
			if ( i <= 8 ) {
				gRomSize = 32*1024;
				gRomSize <<= i;
			}
			else
				gRomSize = 0;
		}
		if ( gCartType > 0 && gCartType < 4 && IN5BUF[0x48] > 4 )
			gMBC1Exp = 1;
		else
			gMBC1Exp = 0;
	}
	else {
		gCartType = 0;
		gRomSize = 0;
		gMBC1Exp = 0;
	}
	gEEPRomType = 0;
	init_port();
	out_adr_data(0x0000, 0x98);		// Read Query
	for ( i = 0; i < 128; i += 2 ) {
		set_adr((unsigned int)i);
		IN5BUF[i/2] = read_data();
	}
	if ( IN5BUF[0] == 0x89 && IN5BUF[1] == 0x15 &&
		IN5BUF[0x10] == 'Q' && IN5BUF[0x11] == 'R' && IN5BUF[0x12]== 'Y' ) {
		gEEPRomType = 64;
		gRomSize = FLASH64M_SIZE;
		gRomBlkSize = FLASH64M_BLOCK;
		out_adr_data(0x0000, 0xff);	// read array
		return;
   }
}

void set_adr_size(unsigned char len)
{
	gFileSize -= len / 4;
	gAddress += len;
}

void read_eeprom()
{
	unsigned char len, i;

	split_adr(gAddress);
	if ( gMBC1Exp ) {
		set_adr(0x6000);
		write_data(0);			// for MCB1 expand bank
		if ( (h & 0x1F) == 0 ) {
			set_adr(0x4000);
			write_data((h>>5) & 0x3);	// use sram bank intend rom bank
		}
		h &= 0x1F;
	}
	set_adr_to_bank();
	set_ai_data(2, 0x81);		// enable read inc.
	set_ai(3);					// read/write data

	len = 0x40 - (l & 0x3F);
	if ( gFileSize < len / 4 ) len = gFileSize * 4;
	for ( i = 0; i < len; i++ )
		IN3BUF[i] = read_data();
	IN3BC = len;
	set_adr_size(len);
	if ( gFileSize == 0 ) {
		set_ai_data(2,0x80);	// disable inc
		gUpLoadFlag = STATUS_DOWNLOAD;
	}
}

unsigned char intel_read_status(void)
{
	out_adr_data(0,0x70);		// read status command
	return read_data();
}

char intel_block_erase(void)
{
	unsigned int time_out = 0x8000;
	while ( intel_read_status() != 0x80 )
		if ( --time_out == 0 )
			return -1;
	gAddress &= ~(FLASH64M_BLOCK-1);
	out_adr_data(gAddress , 0x20);	// block erase
	out_adr_data(gAddress , 0xD0);	// write confirm
	time_out = 0x8000;
	while ( !(intel_read_status() & 0x80) )
		if ( --time_out == 0 ) {
			out_adr_data(gAddress ,0x50);	// clear status register
			return -1;
		}
	if ( (intel_read_status()) == 0x80 )
		return 0;
	out_adr_data(gAddress , 0x50);	// clear status register
	out_adr_data(0x0000, 0xFF);	// read array
	return -1;
}

char intel_buffer_write_32(unsigned char high)
{
	unsigned char i, len;
	unsigned int time_out = 0x8000;
	if ( gFileSize > 8 )
		len = 32;
	else
		len = (unsigned char)gFileSize * 4;

	out_adr_data(gAddress, 0xE8);	// Write buffer command
	set_ai_data(2, 0x82);			// wei enable
	set_ai(3);						// default write mode
	while ( !(read_data() & 0x80) ) {
		if ( --time_out == 0 )
			return -1;
		write_data(0xE8);			// write buffer command
	}
	write_data(len-1);				// set write byte count command
	set_ai_data(2, 0x83);			// enable flash write inc.
	set_ai(3);
	for ( i = 0; i < len; i++ )
		write_data(OUT4BUF[high ? i+32 : i]);
	write_data(0xD0);				// write confirm command
	set_adr_size(len);
	if ( gFileSize  == 0 ) {
		time_out = 0x8000;
		while ( !(intel_read_status() & 0x80) )
			if ( --time_out == 0 ) {
				out_adr_data(0, 0x50);	// clear status register
				return -1;
			}
		if ( intel_read_status() != 0x80 )
			return -1;
		out_adr_data(0, 0xFF);		// read array
	}
	return 0;
}

void access_block()
{
	unsigned char i;
	if ( gUpLoadFlag ) {
		for ( i = 0; i < 0x40; i++ )
			IN3BUF[i] = read_data();
		IN3BC = 0x40;
	}
	else {
		for ( i = 0; i < 0x40; i ++ )
			write_data(OUT4BUF[i]);
	}
	set_adr_size(0x40);
	if ( gFileSize == 0 ) {
		set_ai_data(2,0x80);	// disable inc
		gUpLoadFlag = STATUS_DOWNLOAD;
	}
}

void enable_sram_bank()
{
	init_port();
	set_adr(0x0000);			// write 0000:0x0a default read mode
	out_byte(0x0a);				// enable sram
	out_byte(0xc0);				// disable sram
	set_adr(0xa000);
	out_byte(0xa0);				// ctr index
	set_adr(0xa100);
	// out_byte(0x00);			// ram_off,ram_bank_disable,MBC1
	out_byte(0xc0);				// ram_on,ram_bank_enable,MBC1

	set_adr(0x0000);			// write 0000:0x0a
	out_byte(0x0a);				// enable sram
}

void access_sram()
{
	unsigned char len, i;

	enable_sram_bank();
	l = gAddress & 0xFC;
	m = (gAddress >> 8) & 0x1F;
	h = gAddress >> 13;

	set_adr(0x4000);
	write_data(h);				// set sram bank
	set_ai_data(1, (0xA0 + m));	// sram at 0xa000~bfff
	set_ai_data(0, l);			// a[7..0]=0
	set_ai_data(2, 0x81);		// enable read inc.
	set_ai(3);					// read/write data

	len = 0x40 - (l & 0x3F);
	if ( gFileSize < 0x10 ) len = gFileSize * 4;
	if ( gUpLoadFlag ) {
		for ( i = 0; i < len; i++ )
			IN3BUF[i] = read_data();
		IN3BC = len;
	}
	else {
		for ( i = 0; i < len; i++ )
			write_data(OUT4BUF[i]);
	}
	set_adr_size(len);
	if ( gFileSize == 0 ) {
		set_ai_data(2,0x80);	// disable inc
		gUpLoadFlag = STATUS_DOWNLOAD;
	}
}

char check_port_xpp(void)
{
	init_port();
	set_ai_data(1, 0x12);
	set_ai_data(0, 0x34);
	set_ai(1);
	if ( read_data() != 0x12 ) return 1;
	set_ai(0);
	if ( read_data() != 0x34 ) return 1;
	return 0;
}


//
//  set file size (file size +3) /4
//
void fileSizeSet()
{
	gFileSize = (unsigned long)OUT4BUF[5];
	gFileSize += (unsigned long)OUT4BUF[6] << 8;
	gFileSize += (unsigned long)OUT4BUF[7] << 16;
	gFileSize += (unsigned long)OUT4BUF[8] << 24;

	gFileSize = (gFileSize + 3) / 4;
}
//

void addressSet()
{
	gAddress = (unsigned long)OUT4BUF[1];
	gAddress += (unsigned long)OUT4BUF[2] << 8;
	gAddress += (unsigned long)OUT4BUF[3] << 16;
	gAddress += (unsigned long)OUT4BUF[4] << 24;
}

//
// EndPoint Handler should be modified by each application
//
//
// INEP1:Interrupt In #1
//
/*
void InEP01(void)
{
	unsigned char p, count;

	if (IN1CS & INnCS_BSY)			// Busy on transmitting
		return;

	if (EPCTRL.iep_count) {
		p = 0;
		count = EPCTRL.iep_count;
		while (count) {

			// Your routine is here
			
			// End of your routine

			// You have to modify following step value !!
			count = count - BUFFER_HANDLING_UNIT_EP01;
			p = p + BUFFER_HANDLING_UNIT_EP01;
		}
		IN1BC = EPCTRL.iep_count;	// BSY = 1
	}

}
*/

//
// OUTEP2:BulkOut EndPoint #2
//
/*
void OutEP02(void)
{
	if (OUT2CS & INnCS_BSY)			// Busy on transmitting
		return;

	if (OUT2BC) {

		// *** Your routine is here ***


		// *** End of your routine ***

		OUT2BC = EPSIZE ;	// BSY = 1
	}

}
*/

//
// INEP3:BulkIn EndPoint #3
//
void InEP03(void)
{
	if (IN3CS & INnCS_BSY)			// Busy on transmitting
		return;

	if (IN3BC) {
		// *** Your routine is here ***
		if ( gUpLoadFlag == STATUS_UPLOAD ) {
			OEA = RLED_ON;
			if ( (unsigned char)gAddress && gFileSize >= 0x10 )
				access_block();
			else if ( gAccessType )
				access_sram();
			else
				read_eeprom();
			OEA = LED_OFF;
		}

		// *** End of your routine ***
//		IN3BC = EPSIZE;	// BSY = 1
	}
}

//
//
//
// OUTEP4:BulkOut EndPoint #4
//
void OutEP04(void)
{
	if (OUT4CS & INnCS_BSY)			// Busy on transmitting
		return;

	// *** Your routine is here ***
	switch(OUT4BC)
	{
	case 0: break;
	case COMMAND_SIZE:
		switch(OUT4BUF[0]){
		case GBXCH_CART_PROBE:
			OEA = RLED_ON;
			init_port();
			check_eeprom();
			IN3BUF[0] = gEEPRomType;
			IN3BUF[1] = 0;
			IN3BUF[2] = gCartType;
			IN3BUF[3] = 0;
			IN3BUF[4] = (unsigned char)gRomSize;
			IN3BUF[5] = (unsigned char)(gRomSize >> 8);
			IN3BUF[6] = (unsigned char)(gRomSize >> 16);
			IN3BUF[7] = (unsigned char)(gRomSize >> 24);
			IN3BUF[8] = (unsigned char)gRomBlkSize;
			IN3BUF[9] = (unsigned char)(gRomBlkSize >> 8);
			IN3BUF[10] = (unsigned char)(gRomBlkSize >> 16);
			IN3BUF[11] = (unsigned char)(gRomBlkSize >> 24);
			IN3BC = RESPONSE_SIZE;
			OUT4BC = EPSIZE;		// BSY = 1
			break;

		case GBXCH_ROM_BERASE:
			OEA = WLED_ON;
			addressSet();
			switch ( gEEPRomType ) {
			case 64:
				IN3BUF[0] = intel_block_erase();
				break;
			default :
				IN3BUF[0] = -1;
			}
			IN3BUF[1] = 0;
			IN3BUF[2] = 0;
			IN3BUF[3] = 0;
			IN3BC = RESPONSE_SIZE;
			OUT4BC = EPSIZE ;		// BSY = 1
			break;

		case GBXCH_ROM_WRITE:
			OEA = WLED_ON;
			addressSet();
			fileSizeSet();
			gUpLoadFlag = STATUS_DOWNLOAD;
			gAccessType = ACCESS_ROM;
			gErase = 0;
			IN3BUF[0] = 0;
			OUT4BC = EPSIZE ;		// BSY = 1
			break;

		case GBXCH_ROM_BWRITE:
			OEA = WLED_ON;
			addressSet();
			fileSizeSet();
			gUpLoadFlag = STATUS_DOWNLOAD;
			gAccessType = ACCESS_ROM;
			gErase = 1;
			IN3BUF[0] = 0;
			OUT4BC = EPSIZE ;		// BSY = 1
			break;

		case GBXCH_SRAM_WRITE:
			OEA = WLED_ON;
			addressSet();
			fileSizeSet();
			gUpLoadFlag = STATUS_DOWNLOAD;
			gAccessType = ACCESS_SRAM;
			gWriteFirst = 1;
			OUT4BC = EPSIZE ;		// BSY = 1
			break;

		case GBXCH_READ:
			OEA = RLED_ON;
			addressSet();
			fileSizeSet();
			gUpLoadFlag = STATUS_UPLOAD;
			gAccessType = ACCESS_ROM;
			read_eeprom();
			OUT4BC = EPSIZE ;		// BSY = 1
			break;

		case GBXCH_SRAM_READ:
			OEA = RLED_ON;
			addressSet();
			fileSizeSet();
			gUpLoadFlag = STATUS_UPLOAD;
			gAccessType = ACCESS_SRAM;
			access_sram();
			OUT4BC = EPSIZE ;		// BSY = 1
			break;

		case TARGET_INIT:
			reset_port();
			IN3BUF[0] = check_port_xpp();
			IN3BUF[1] = 0;
			IN3BUF[2] = 0;
			IN3BUF[3] = 0;
			IN3BC = RESPONSE_SIZE;
			OUT4BC = EPSIZE ;		// BSY = 1
			break;

		case TARGET_END:
			OEA = 0;
			OEB = 0;
			OEC = 0;
			OUTA = 0;
			OUTB = 0;
			OUTC = 0;
			OUT4BC = EPSIZE ;		// BSY = 1
			break;

		default:
			OUT4BC = EPSIZE ;		// BSY = 1
			break;
		}
		break;

	default : //64
		if ( gFileSize ) {
			OEA = WLED_ON;
			if ( gAccessType ) {
				if ( (unsigned char)gAddress && gFileSize >= 0x10 && gWriteFirst == 0 )
					access_block();
				else
					access_sram();
			}
			else {
				switch ( gEEPRomType ) {
				case 64:
					if ( gErase && (gAddress & (FLASH64M_BLOCK-1)) == 0 )
						IN3BUF[0] = intel_block_erase();
					if ( IN3BUF[0] == 0 )
						IN3BUF[0] = intel_buffer_write_32(0);
					if ( gFileSize && IN3BUF[0] == 0 )
						IN3BUF[0] = intel_buffer_write_32(1);
					break;
				}
				if ( IN3BUF[0] ) gFileSize = 0;
				if ( gFileSize == 0 ) {
					IN3BUF[1] = 0;
					IN3BUF[2] = 0;
					IN3BUF[3] = 0;
					IN3BC = RESPONSE_SIZE;
				}
			}
		}
		OUT4BC = EPSIZE ;			// BSY = 1
		gWriteFirst = 0;
		break;
	}
	OEA = LED_OFF;
	// *** End of your routine ***
}

//
//

//
// INEP5:BulkIn EndPoint #5
//
/*
void InEP05(void)
{
	if (IN5CS & INnCS_BSY)			// Busy on transmitting
		return;

	if (IN5BC) {

		// *** Your routine is here ***

		// *** End of your routine ***

		IN5BC = EPSIZE;	// BSY = 1
	}
}
*/

/////////////////////////////// Will not be changed /////////////////////////////////////

void usb_disconnect(void)
{
	unsigned char d;
	d = USBCS & ~(USBCS_DISCON|USBCS_DISCOE);
	d |= USBCS_DISCOE | USBCS_DISCON;		// DISOCON# Output Enable & DATA=0
	USBCS = d;
}


void usb_connect(void)
{
	unsigned char d;
	d = USBCS & ~(USBCS_DISCON|USBCS_DISCOE);
	d |= USBCS_DISCOE | USBCS_RENUM;		// DISOCON# Output Enable & DATA=1
	USBCS = d;
}


//
// main routine
//
void main(void)
{
	IE   = 0x00;	// Interrupt Disable

	usb_disconnect();
	register_init();
	setup_usb_inf();
	usb_connect();

	IE   = 0x80;	// Interrupt Enable
	gUpLoadFlag = STATUS_DOWNLOAD;

	/* wait for smething */
	while(1){
	}
}


//
// Response for GET_DESCRIPTOR
//
void exec_get_descriptor(unsigned char type)
{
	unsigned int	adrs;
	switch(type) {
		case 0x01:	// Device Descriptor
			adrs = (unsigned int)(&IN7BUF[0]);
			SUDPTRH = ((adrs >> 8)  & 0xff);
			SUDPTRL = (adrs & 0xff);
		break;

		case 0x02:	// Configuration Descriptor
			adrs = (unsigned int)(&IN7BUF[DEVICE_DISC_LEN]);
			SUDPTRH = ((adrs >> 8) & 0xff);
			SUDPTRL = adrs & 0xff;
		break;

		case 0x03:	// String Descriptor
			EPCTRL.cep_stat = STS_STALL;
		break;

		default:
			EPCTRL.cep_stat = STS_STALL;
		break;
	}
}

//
// USB starndard request
//
void exec_std_request(void)
{
	switch(SETUPDAT[1]) {
	case 0x06:	// GET_DESCRIPTOR
			exec_get_descriptor(SETUPDAT[3]);
	break;

//	case 0x09:

	default:
		break;
	}
}

//void usb_int(void) interrupt 8
void usb_int(void) interrupt 8
{
	unsigned char type;

	if(IVEC==0x3c)
	{
		OutEP04();
		EXIF &= ~EXIF_USB;
		OUT07IRQ = 0x10;
	}
	else
        switch(IVEC) {
		case 0x00:				// Setup Data Valid
			EPCTRL.cep_stat = STS_READY;
			type = SETUPDAT[0] & 0x60;
			switch(type) {
			case 0x00:		// Standarad
				exec_std_request();

				//
				// Hint:
				// You have to put on your own handler as IN-EndPoint Handler
				// In normal case, you should not put your owns at here.
				// However, EZ-USB(AN21xx) can not handle by corresponding interrupt
				//

				//
/*				EPCTRL.iep_count = INIT_EP01_COUNT;
				InEP01();

				OUT2BC = EPSIZE;
*/				break;

			default:		// Class & Reserved
				break;
			}
			if (EPCTRL.cep_stat == STS_READY) {
				EP0CS  |= EP0CS_HSNAK;
			} else {
				EP0CS  |= (EP0CS_HSNAK | EP0CS_STALL);
			}
            EXIF &= ~EXIF_USB;
			USBIRQ |= USBIRQ_SUDAVIR;
			break;
/*
		case 0x20:		// EP1-IN (Interrupt-IN Endpoint)
			InEP01();
			EXIF &= ~EXIF_USB;
			IN07IRQ = 0x02;
			break;

		case 0x2c:		// EP2-OUT (Bulk-OUT Endpoint)
			OutEP02();
			EXIF &= ~EXIF_USB;
			OUT07IRQ = 0x04;
			break;
*/
		case 0x30:		// EP3-IN (Bulk-IN Endpoint)
			InEP03();
			EXIF &= ~EXIF_USB;
			IN07IRQ = 0x08;
			break;

		case 0x3c:		// EP4-OUT(Bulk-OUT Endpoint)
			OutEP04();
			EXIF &= ~EXIF_USB;
			OUT07IRQ = 0x10;
			break;
/*
		case 0x40:		// EP5-IN (Bulk-IN  Endpoint)
			InEP05();
			EXIF &= ~EXIF_USB;
			IN07IRQ  = 0x20;
			break;
*/
		default:
			EXIF &= ~EXIF_USB;
			USBIRQ |= USBIRQ_SUDAVIR;
			break;
		}
}
